FillablePalletLoader = {};

function FillablePalletLoader.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(BaleLoader, specializations);
end;

function FillablePalletLoader:load(savegame)
    self.loadPalletsToo = true;
    for _,specialization in pairs(self.specializations) do
        if specialization.GRAB_DROP_BALE then
            specialization.draw = function() end;
        end;
    end;
end;

function FillablePalletLoader:delete()
end;

function FillablePalletLoader:mouseEvent(posX, posY, isDown, isUp, button)
end;

function FillablePalletLoader:keyEvent(unicode, sym, modifier, isDown)
end;

function FillablePalletLoader:update(dt)
end;

function FillablePalletLoader:updateTick(dt)
end;

function FillablePalletLoader:draw()
    if self:getIsActiveForInput(true) then
        if self.emptyState == BaleLoader.EMPTY_NONE then
            if self.grabberMoveState == nil then
                if self.isInWorkPosition then
                    g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_TRANSPORT"), self.workTransportButton);
                else
                    g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_WORK"), self.workTransportButton);
                end;
            end;
            if BaleLoader.getAllowsStartUnloading(self) then
                g_currentMission:addHelpButtonText(g_i18n:getText("PALLETLOADER_UNLOAD"), self.emptyButton);
            end;
        else
            if self.emptyState >= BaleLoader.EMPTY_TO_WORK and self.emptyState <= BaleLoader.EMPTY_ROTATE2 then
                g_currentMission:addExtraPrintText(g_i18n:getText("BALELOADER_UP"));
            elseif self.emptyState == BaleLoader.EMPTY_CANCEL or self.emptyState == BaleLoader.EMPTY_SINK then
                g_currentMission:addExtraPrintText(g_i18n:getText("BALELOADER_DOWN"));
            elseif self.emptyState == BaleLoader.EMPTY_WAIT_TO_DROP then
                g_currentMission:addHelpButtonText(g_i18n:getText("PALLETLOADER_READY"), self.emptyButton);
                g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_ABORT"), self.emptyAbortButton);
            elseif self.emptyState == BaleLoader.EMPTY_WAIT_TO_SINK then
                g_currentMission:addHelpButtonText(g_i18n:getText("BALELOADER_SINK"), self.emptyButton);
            elseif self.emptyState == BaleLoader.EMPTY_WAIT_TO_REDO then
                g_currentMission:addHelpButtonText(g_i18n:getText("PALLETLOADER_UNLOAD"), self.emptyButton);
            end;
        end;
    end;
    if self.showInvalidBaleWarning then
        g_currentMission:showBlinkingWarning(g_i18n:getText("InvalidPalletWarning"));
    end;
end;

function BaleLoader.getBaleInRange(self, refNode)
    local px, py, pz = getWorldTranslation(refNode);
    local nearestDistance = 3.0;
    local nearestBale = nil;
    local nearestBaleType = nil;
    for index, item in pairs(g_currentMission.itemsToSave) do
        local bale = item.item;
        if bale:isa(Bale) or (bale:isa(FillablePallet) and self.loadPalletsToo) then
            local vx, vy, vz = getWorldTranslation(bale.nodeId);
            local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
            if distance < nearestDistance then
                local foundBaleType;
                for _, baleType in pairs(self.allowedBaleTypes) do
                    if baleType.minBaleDiameter ~= nil then
                        if bale.baleDiameter ~= nil and bale.baleWidth ~= nil and
                           bale.baleDiameter >= baleType.minBaleDiameter and bale.baleDiameter <= baleType.maxBaleDiameter and
                           bale.baleWidth >= baleType.minBaleWidth and bale.baleWidth <= baleType.maxBaleWidth
                        then
                            foundBaleType = baleType;
                            break;
                        end
                    else
                        if bale.baleWidth ~= nil and bale.baleHeight ~= nil and bale.baleLength ~= nil and
                           bale.baleWidth >= baleType.minBaleWidth and bale.baleWidth <= baleType.maxBaleWidth and
                           bale.baleHeight >= baleType.minBaleHeight and bale.baleHeight <= baleType.maxBaleHeight and
                           bale.baleLength >= baleType.minBaleLength and bale.baleLength <= baleType.maxBaleLength
                        then
                            foundBaleType = baleType;
                            break;
                        end
                    end
                end
                if foundBaleType ~= nil or nearestBaleType == nil then
                    if foundBaleType ~= nil then
                        nearestDistance = distance;
                    end
                    nearestBale = bale;
                    nearestBaleType = foundBaleType;
                end
            end
        end;
    end;
    return nearestBale, nearestBaleType;
end;
